local noop = function() end

local BusterAttack = require("com.Dawn.EnemyBusterAction")

local move_texture = Resources.load_texture("mob_move.png")

function character_init(dark_rock)
    --meta
    dark_rock:set_name("!?")

    local entity_query = function(ent)
        return Obstacle.from(ent) ~= nil and Character.from(ent) ~= nil and Player.from(ent) ~= nil;
    end

    dark_rock.can_move_to_func = function(tile)
        if not tile then return false end
        --Tile exists, tile team is compatible, tile is not an edge tile, and no matching entities exist on tile?
        --Then he can move to it.
        return dark_rock:is_team(tile:team()) and not tile:is_edge() and #tile:find_entities(entity_query) == 0
    end

    local anim = nil;
    local field = nil;

    local target = nil;
    local new_tile = nil;
    local target_tile = nil;

    local attack_once = false;
    local can_move = true;

    local charge_threshold = 0;
    local charge_counter = 0;

    dark_rock.on_battle_start_func = noop
    dark_rock.on_battle_end_func = noop

    local function find_player(self)
        local query = function(e)
            return true;
        end
        local potential_threats = field:find_players(query);
        if #potential_threats > 0 then
            return potential_threats[1]
        end
        return nil
    end

    dark_rock.on_spawn_func = function(self)
        field = self:field();
        target = find_player(self)
        if target ~= nil then
            self:set_health(target:max_health())
            self:set_texture(target:texture())
            local palette = target:palette()
            if palette ~= nil then
                self:set_palette(palette)
            end
            anim = self:animation();
            local target_anim = target:animation()
            anim:copy_from(target_anim)
            anim:set_state(target_anim:state())
            anim:set_playback(Playback.Loop)
            target_tile = target:current_tile();
            charge_threshold = target:calculate_charge_time();
        end
    end

    local function spawn_mob_move(x, y)
        local mob_tile = field:tile_at(x, y)
        local fx = Artifact.new();
        fx:set_texture(move_texture)
        local fx_anim = fx:animation()
        fx_anim:load("mob_move.animation")
        fx_anim:set_state("DEFAULT")
        fx_anim:apply(fx:sprite())
        fx_anim:on_complete(function()
            fx:erase()
        end)
        field:spawn(fx, mob_tile)
    end

    local mirrored_movement = function(self, other)
        new_tile = other:current_tile();
        if new_tile ~= target_tile then
            local x_offset = (target_tile:x() - new_tile:x())

            local own_tile = self:current_tile();
            local own_x = own_tile:x()
            local own_y = own_tile:y()

            local dest = field:tile_at(own_x + x_offset, new_tile:y())
            self:teleport(dest, function()
                spawn_mob_move(own_x, own_y);
                target_tile = new_tile;
            end)
        end
    end

    local mirror_attack = function(self, other)
        local target_card = other:field_card(1);

        if target_card ~= nil then
            self:queue_action(Action.from_card(self, target_card));
        end

        return nil
    end

    local buster_attack = function(self, other, is_charge)
        charge_counter = 0;
        can_move = false;
        if is_charge then
            self:queue_action(BusterAttack.new(other, is_charge, other:attack_level()*10, self))
        else
            self:queue_action(BusterAttack.new(other, is_charge, other:attack_level(), self))
        end
    end

    dark_rock.on_update_func = function(self)
        if target:deleted() then return end
        if target:input_has(Input.Held.Shoot) then
            charge_counter = charge_counter + 1;
        end
        if not target:input_has(Input.Held.Shoot) and charge_counter > 0 then
            buster_attack(self, target, charge_counter >= charge_threshold);
        end
        if can_move then
            mirrored_movement(self, target)
        elseif anim:state() == "PLAYER_IDLE" then
            can_move = true
        end
        if target:input_has(Input.Pressed.Use) then
            attack_once = true;
        end
        if attack_once == true then
            mirror_attack(self, target);
            attack_once = false;
        end
    end

    dark_rock.on_delete_func = function(self)
        self:default_character_delete();
    end
end
